/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "sfvfile.h"

#include <zlib.h>

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qregexp.h>

namespace KDigest {

/**
@brief Default constructor
@param path Path of the SFV file
@param parent parent object (passed to DigestFile and then to QObject)
@param name object's name (passed to DigestFile and then to QObject)
*/
SFVfile::SFVfile(const QString &path, QObject *parent, const char *name)
	: DigestFile(parent, name)
{
	QFile f(path);
	if ( ! f.open(IO_ReadOnly) ) {
		///@todo Thrown an error
		return;
	}
	
	QTextStream ts(&f);
	
	QString line;
	QRegExp digest("^(.*) ([0-9a-fA-F]{1,8})[ ]*$");
	QRegExp comment("^[ ]*;(.*)$");
	if ( path.right(3).lower() == ".afv" )		// This is an AFV file,
		comment.setPattern("^[ ]*\\*(.*)$");	// changing the comment style
	while(!ts.atEnd())
	{
		line = ts.readLine();
		if ( line == QString::null )
			continue;
		
		if ( comment.search(line) != -1 )
			m_comments += comment.cap(1) + "\n";
		
		if ( digest.search(line) == -1 )
			continue;
		
		SFVentry *e = new SFVentry;
		
		e->filename = digest.cap(1);
		e->status = dsNotChecked;
		
		e->crc32 = digest.cap(2).toUInt(NULL, 16);
		e->aCrc32 = 0;
		
		m_entries.append(e);
	}
	
	f.close();
}

SFVfile::~SFVfile()
{
}

/**
@brief Saves the data in the SFVfile in the given filename
@param filename Complete path where to save the new digest
@note This function differs from DigestFile::saveDigest() by saving also the comment
*/
bool SFVfile::saveDigest(const QString &filename)
{
	QFile newDigest(filename);
	if ( ! newDigest.open(IO_WriteOnly) )
	{
		qWarning("Unable to open file for writing");
		return false;
	}
	
	QStringList comments = QStringList::split("\n", m_comments);
	
	for( QStringList::iterator it = comments.begin(); it != comments.end(); it++ )
	{
		QString comment = ";" + (*it) + "\n";
		newDigest.writeBlock(comment, comment.length());
	}
	
	for( DigestEntries::iterator it = m_entries.begin(); it != m_entries.end(); it++ )
		(*it)->writeOnFile(newDigest);
	
	newDigest.close();
	
	return true;
}

/**
@brief Adds a file in the digest
@param filename Complete path of the file to add
*/
void SFVfile::addFile(const QString &filename)
{
	m_entries.append( new SFVentry( filename ) );
}

/**
@brief Gets the digest of the given opened file
@param f The file to get the digest of (opened)
*/
inline const uint32_t SFVfile::SFVentry::getDigest(QFile &f)
{
	int read = 0;
	uint32_t crc = ::crc32(0L, Z_NULL, 0);
	
	while( (read = f.readBlock((char*)m_buffer, 4096) ) )
	{
		crc = ::crc32(crc, m_buffer, read);
		qApp->processEvents();
	}
	
	return crc;
}

/**
@brief Create a new SFVentry creating the digest for the given file
@param filepath Complete path of the file to digest
*/
SFVfile::SFVentry::SFVentry(const QString &filepath)
{
	QDir filedir(filepath); filedir.cd("..");
	
	QFile f(filepath);
	if ( ! f.open(IO_ReadOnly) )
	{
		status = dsErrorAccess;
		return;
	}
	
	filename = filepath; filename.remove(filedir.path() + "/");
	crc32 = getDigest(f);

	f.close();	
}

/**
@brief Do the dirt work on calculating the digest of a file
@param filename Complete filename of the entry
*/
bool SFVfile::SFVentry::calculateActual(const QString &filename)
{
	QFile chkFile(filename);
	if ( ! chkFile.open(IO_ReadOnly) )
	{
		status = dsErrorAccess;
		return false;
	}
	
	aCrc32 = getDigest(chkFile);
	
	if ( aCrc32 == crc32 )
	{
		status = dsOk;
		return true;
	} else {
		status = dsBadDigest;
		return false;
	}
	
	chkFile.close();
}

/**
@brief Write the entry on the given file
@param file Opened file where to save the entry to
*/
void SFVfile::SFVentry::writeOnFile(QFile &file)
{
	QString entryLine = filename + " " + digest() + "\n";
	file.writeBlock(entryLine, entryLine.length());
}

};
